/***************************************************************************
 *   Copyright (C) 2008 by Tobias Koenig <tokoe@kde.org>                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef FULLVIEWWIDGET_H
#define FULLVIEWWIDGET_H

#include <QtGui/QImage>
#include <QtGui/QWidget>

class QWheelEvent;

class FullViewWidget : public QWidget
{
    public:
        FullViewWidget();
        ~FullViewWidget();

        void setImage( const QImage &image );
        void adaptPosition( const QPoint &pos, int screenId );

    protected:
        virtual void paintEvent( QPaintEvent* );
        virtual void mousePressEvent( QMouseEvent* );
        virtual void wheelEvent( QWheelEvent* );

    private:
        QImage mImage;
        QRect mDesktopSize;
};

#endif
